"""rate contracts

Revision ID: 071e8f29d257
Revises: 0363f582ab28
Create Date: 2021-09-11 05:57:49.062468

"""
import sqlalchemy as sa

from alembic import op

# revision identifiers, used by Alembic.
from brewman.models.permission import Permission
from sqlalchemy.dialects import postgresql


revision = "071e8f29d257"
down_revision = "0363f582ab28"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.execute(Permission.__table__.insert().values(id="f97589c6-df9f-45d3-874d-4245a4d99ade", name="Rate Contracts"))
    op.create_table(
        "rate_contracts",
        sa.Column("id", postgresql.UUID(as_uuid=True), nullable=False),
        sa.Column("date", sa.Date(), nullable=False),
        sa.Column("vendor_id", postgresql.UUID(as_uuid=True), nullable=False),
        sa.Column("valid_from", sa.Date(), nullable=True),
        sa.Column("valid_till", sa.Date(), nullable=True),
        sa.Column("narration", sa.Unicode(length=1000), nullable=False),
        sa.Column("user_id", postgresql.UUID(as_uuid=True), nullable=False),
        sa.Column("creation_date", sa.DateTime(), nullable=False),
        sa.Column("last_edit_date", sa.DateTime(), nullable=False),
        sa.ForeignKeyConstraint(["user_id"], ["users.id"], name=op.f("fk_rate_contracts_user_id_users")),
        sa.ForeignKeyConstraint(["vendor_id"], ["accounts.id"], name=op.f("fk_rate_contracts_vendor_id_accounts")),
        sa.PrimaryKeyConstraint("id", name=op.f("pk_rate_contracts")),
    )
    op.create_index(op.f("ix_rate_contracts_date"), "rate_contracts", ["date"], unique=False)
    op.create_table(
        "rate_contract_items",
        sa.Column("id", postgresql.UUID(as_uuid=True), nullable=False),
        sa.Column("rate_contract_id", postgresql.UUID(as_uuid=True), nullable=False),
        sa.Column("product_id", postgresql.UUID(as_uuid=True), nullable=False),
        sa.Column("cost_price", sa.Numeric(precision=15, scale=2), nullable=False),
        sa.ForeignKeyConstraint(
            ["product_id"], ["products.id"], name=op.f("fk_rate_contract_items_product_id_products")
        ),
        sa.ForeignKeyConstraint(
            ["rate_contract_id"],
            ["rate_contracts.id"],
            name=op.f("fk_rate_contract_items_rate_contract_id_rate_contracts"),
        ),
        sa.PrimaryKeyConstraint("id", name=op.f("pk_rate_contract_items")),
        sa.UniqueConstraint("rate_contract_id", "product_id", name=op.f("uq_rate_contract_items_rate_contract_id")),
    )
    op.create_index(
        op.f("ix_rate_contract_items_rate_contract_id"), "rate_contract_items", ["rate_contract_id"], unique=False
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_index(op.f("ix_rate_contract_items_rate_contract_id"), table_name="rate_contract_items")
    op.drop_table("rate_contract_items")
    op.drop_index(op.f("ix_rate_contracts_date"), table_name="rate_contracts")
    op.drop_table("rate_contracts")
    # ### end Alembic commands ###
