"""enums

Revision ID: 7ba0aff64237
Revises: d6f96b7b16c6
Create Date: 2021-10-29 18:26:58.200044

"""
import sqlalchemy as sa

from alembic import op
from sqlalchemy import column, table


# revision identifiers, used by Alembic.
revision = "7ba0aff64237"
down_revision = "d6f96b7b16c6"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    voucher_type = sa.Enum(
        "JOURNAL",
        "PURCHASE",
        "ISSUE",
        "PAYMENT",
        "RECEIPT",
        "PURCHASE_RETURN",
        "OPENING_ACCOUNTS",
        "OPENING_BATCHES",
        "CLOSING_STOCK",
        "OPENING_BALANCE",
        "CLOSING_BALANCE",
        "EMPLOYEE_BENEFIT",
        "INCENTIVE",
        name="voucher_type",
    )
    voucher_type.create(op.get_bind())
    op.add_column(
        "vouchers",
        sa.Column(
            "vt",
            voucher_type,
            nullable=True,
        ),
    )
    v = table("vouchers", column("voucher_type", sa.INTEGER()), column("vt", voucher_type))
    op.execute(v.update().where(v.c.voucher_type == 1).values(vt="JOURNAL"))
    op.execute(v.update().where(v.c.voucher_type == 2).values(vt="PURCHASE"))
    op.execute(v.update().where(v.c.voucher_type == 3).values(vt="ISSUE"))
    op.execute(v.update().where(v.c.voucher_type == 4).values(vt="PAYMENT"))
    op.execute(v.update().where(v.c.voucher_type == 5).values(vt="RECEIPT"))
    op.execute(v.update().where(v.c.voucher_type == 6).values(vt="PURCHASE_RETURN"))
    op.execute(v.update().where(v.c.voucher_type == 7).values(vt="OPENING_ACCOUNTS"))
    op.execute(v.update().where(v.c.voucher_type == 8).values(vt="OPENING_BATCHES"))
    op.execute(v.update().where(v.c.voucher_type == 9).values(vt="CLOSING_STOCK"))
    op.execute(v.update().where(v.c.voucher_type == 13).values(vt="INCENTIVE"))
    op.drop_column("vouchers", "voucher_type")

    op.alter_column("vouchers", "vt", new_column_name="voucher_type", existing_nullable=True, nullable=False)

    op.create_table(
        "account_types",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("name", sa.Unicode(length=255), nullable=False),
        sa.Column("balance_sheet", sa.Boolean(), nullable=False),
        sa.Column("debit", sa.Boolean(), nullable=False),
        sa.Column("cash_flow_classification", sa.Unicode(length=255), nullable=False),
        sa.Column("order", sa.Integer(), nullable=False),
        sa.Column("show_in_list", sa.Boolean(), nullable=False),
        sa.PrimaryKeyConstraint("id", name=op.f("pk_account_types")),
        sa.UniqueConstraint("name", name=op.f("uq_account_types_name")),
    )
    at = table(
        "account_types",
        column("id", sa.Integer()),
        column("name", sa.Unicode(length=255)),
        column("balance_sheet", sa.Boolean()),
        column("debit", sa.Boolean()),
        column("cash_flow_classification", sa.Unicode(length=255)),
        column("order", sa.Integer()),
        column("show_in_list", sa.Boolean()),
    )
    op.execute(
        at.insert().values(
            id=1,
            name="Cash",
            balance_sheet=True,
            debit=True,
            cash_flow_classification="Cash",
            order=10000,
            show_in_list=True,
        )
    )
    op.execute(
        at.insert().values(
            id=2,
            name="Purchase",
            balance_sheet=False,
            debit=True,
            cash_flow_classification="Operating",
            order=20000,
            show_in_list=True,
        )
    )
    op.execute(
        at.insert().values(
            id=3,
            name="Sale",
            balance_sheet=False,
            debit=False,
            cash_flow_classification="Operating",
            order=10000,
            show_in_list=True,
        )
    )
    op.execute(
        at.insert().values(
            id=4,
            name="Assets",
            balance_sheet=True,
            debit=True,
            cash_flow_classification="Investing",
            order=20000,
            show_in_list=True,
        )
    )
    op.execute(
        at.insert().values(
            id=5,
            name="Capital",
            balance_sheet=True,
            debit=False,
            cash_flow_classification="Financing",
            order=70000,
            show_in_list=True,
        )
    )
    op.execute(
        at.insert().values(
            id=6,
            name="Debtors",
            balance_sheet=True,
            debit=True,
            cash_flow_classification="Operating",
            order=30000,
            show_in_list=True,
        )
    )
    op.execute(
        at.insert().values(
            id=7,
            name="Expenses",
            balance_sheet=False,
            debit=True,
            cash_flow_classification="Operating",
            order=40000,
            show_in_list=True,
        )
    )
    op.execute(
        at.insert().values(
            id=9,
            name="Creditors",
            balance_sheet=True,
            debit=False,
            cash_flow_classification="Operating",
            order=60000,
            show_in_list=True,
        )
    )
    op.execute(
        at.insert().values(
            id=10,
            name="Salary",
            balance_sheet=True,
            debit=True,
            cash_flow_classification="Operating",
            order=40000,
            show_in_list=False,
        )
    )
    op.execute(
        at.insert().values(
            id=11,
            name="Liabilities",
            balance_sheet=True,
            debit=False,
            cash_flow_classification="Operating",
            order=50000,
            show_in_list=True,
        )
    )
    op.execute(
        at.insert().values(
            id=12,
            name="Revenue",
            balance_sheet=False,
            debit=False,
            cash_flow_classification="Operating",
            order=30000,
            show_in_list=True,
        )
    )
    op.execute(
        at.insert().values(
            id=13,
            name="Tax",
            balance_sheet=True,
            debit=False,
            cash_flow_classification="Operating",
            order=80000,
            show_in_list=True,
        )
    )
    op.create_foreign_key(op.f("fk_accounts_type_account_types"), "accounts", "account_types", ["type"], ["id"])
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    pass
    # ### end Alembic commands ###
