import uuid

from datetime import date, datetime
from decimal import Decimal
from typing import List, Optional, Union

from pydantic import validator
from pydantic.main import BaseModel

from . import to_camel
from .product import ProductLink


class ProductLedgerItem(BaseModel):
    id_: Optional[uuid.UUID]
    date_: date
    name: str
    url: List[str]
    type_: str
    narration: str
    debit_quantity: Optional[Decimal]
    debit_amount: Optional[Decimal]
    debit_unit: Optional[str]
    credit_quantity: Optional[Decimal]
    credit_amount: Optional[Decimal]
    credit_unit: Optional[str]
    running_quantity: Decimal
    running_amount: Decimal
    posted: bool

    @validator("date_", pre=True)
    def parse_date(cls, value: Union[date, str]) -> date:
        if isinstance(value, date):
            return value
        return datetime.strptime(value, "%d-%b-%Y").date()

    class Config:
        anystr_strip_whitespace = True
        alias_generator = to_camel
        json_encoders = {date: lambda v: v.strftime("%d-%b-%Y")}


class ProductLedger(BaseModel):
    start_date: date
    finish_date: date
    product: Optional[ProductLink]
    body: List[ProductLedgerItem]

    class Config:
        anystr_strip_whitespace = True
        alias_generator = to_camel
        json_encoders = {date: lambda v: v.strftime("%d-%b-%Y")}

    @validator("start_date", pre=True)
    def parse_start_date(cls, value: Union[date, str]) -> date:
        if isinstance(value, date):
            return value
        return datetime.strptime(value, "%d-%b-%Y").date()

    @validator("finish_date", pre=True)
    def parse_finish_date(cls, value: Union[date, str]) -> date:
        if isinstance(value, date):
            return value
        return datetime.strptime(value, "%d-%b-%Y").date()
