import uuid

from datetime import date, datetime
from decimal import Decimal
from typing import List, Optional, Union

from pydantic import BaseModel, validator

from . import to_camel
from .product import ProductLink
from .recipe_item import RecipeItem


class RecipeIn(BaseModel):
    sku: ProductLink

    recipe_yield: Decimal
    cost_price: Decimal
    sale_price: Decimal
    notes: str

    valid_from: date
    valid_till: date

    items: List[RecipeItem]

    class Config:
        anystr_strip_whitespace = True
        alias_generator = to_camel
        json_encoders = {date: lambda v: v.strftime("%d-%b-%Y") if v is not None else None}

    @validator("valid_from", pre=True)
    def parse_valid_from(cls, value: Union[date, str]) -> date:
        if isinstance(value, date):
            return value
        return datetime.strptime(value, "%d-%b-%Y").date()

    @validator("valid_till", pre=True)
    def parse_valid_till(cls, value: Union[date, str]) -> date:
        if isinstance(value, date):
            return value
        return datetime.strptime(value, "%d-%b-%Y").date()


class Recipe(RecipeIn):
    id_: uuid.UUID

    class Config:
        anystr_strip_whitespace = True
        alias_generator = to_camel
        json_encoders = {date: lambda v: v.strftime("%d-%b-%Y")}


class RecipeBlank(RecipeIn):
    sku: Optional[ProductLink]  # type: ignore[assignment]

    class Config:
        anystr_strip_whitespace = True
        alias_generator = to_camel
        json_encoders = {date: lambda v: v.strftime("%d-%b-%Y")}
