import { HttpClient, HttpHeaders } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { Observable } from 'rxjs/internal/Observable';
import { catchError } from 'rxjs/operators';

import { ErrorLoggerService } from '../core/error-logger.service';

import { Client } from './client';

const url = '/api/clients';
const serviceName = 'ClientService';

@Injectable({
  providedIn: 'root',
})
export class ClientService {
  constructor(private http: HttpClient, private log: ErrorLoggerService) {}

  get(id: string | null): Observable<Client> {
    const getUrl: string = id === null ? url : `${url}/${id}`;
    return this.http
      .get<Client>(getUrl)
      .pipe(catchError(this.log.handleError(serviceName, `get id=${id}`))) as Observable<Client>;
  }

  list(): Observable<Client[]> {
    return this.http
      .get<Client[]>(`${url}/list`)
      .pipe(catchError(this.log.handleError(serviceName, 'list'))) as Observable<Client[]>;
  }

  update(client: Client): Observable<Client> {
    return this.http
      .put<Client>(`${url}/${client.id}`, client)
      .pipe(catchError(this.log.handleError(serviceName, 'update'))) as Observable<Client>;
  }

  delete(id: string): Observable<Client> {
    return this.http
      .delete<Client>(`${url}/${id}`)
      .pipe(catchError(this.log.handleError(serviceName, 'delete'))) as Observable<Client>;
  }
}
