import { HttpClient, HttpHeaders } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { Observable } from 'rxjs/internal/Observable';
import { catchError } from 'rxjs/operators';

import { CostCentre } from '../core/cost-centre';
import { ErrorLoggerService } from '../core/error-logger.service';

const url = '/api/cost-centres';
const serviceName = 'CostCentreService';

@Injectable({
  providedIn: 'root',
})
export class CostCentreService {
  constructor(private http: HttpClient, private log: ErrorLoggerService) {}

  get(id: string | null): Observable<CostCentre> {
    const getUrl: string = id === null ? `${url}` : `${url}/${id}`;
    return this.http
      .get<CostCentre>(getUrl)
      .pipe(
        catchError(this.log.handleError(serviceName, `get id=${id}`)),
      ) as Observable<CostCentre>;
  }

  list(): Observable<CostCentre[]> {
    return this.http
      .get<CostCentre[]>(`${url}/list`)
      .pipe(catchError(this.log.handleError(serviceName, 'list'))) as Observable<CostCentre[]>;
  }

  save(costCentre: CostCentre): Observable<CostCentre> {
    return this.http
      .post<CostCentre>(`${url}`, costCentre)
      .pipe(catchError(this.log.handleError(serviceName, 'save'))) as Observable<CostCentre>;
  }

  update(costCentre: CostCentre): Observable<CostCentre> {
    return this.http
      .put<CostCentre>(`${url}/${costCentre.id}`, costCentre)
      .pipe(catchError(this.log.handleError(serviceName, 'update'))) as Observable<CostCentre>;
  }

  saveOrUpdate(costCentre: CostCentre): Observable<CostCentre> {
    if (!costCentre.id) {
      return this.save(costCentre);
    }
    return this.update(costCentre);
  }

  delete(id: string): Observable<CostCentre> {
    return this.http
      .delete<CostCentre>(`${url}/${id}`)
      .pipe(catchError(this.log.handleError(serviceName, 'delete'))) as Observable<CostCentre>;
  }
}
