import { DataSource } from '@angular/cdk/collections';
import { MatPaginator } from '@angular/material/paginator';
import { MatSort } from '@angular/material/sort';
import { SortDirection } from '@angular/material/sort/sort-direction';
import { Observable, of as observableOf } from 'rxjs';

import { Entries } from './entries';

export class EntriesDatasource extends DataSource<Entries> {
  constructor(
    public data: Entries[],
    public length: number,
    private paginator?: MatPaginator,
    private sort?: MatSort,
    private pageSize?: number,
    private pageIndex?: number,
    private activeSort?: string,
    private sortDirection?: SortDirection,
  ) {
    super();
  }

  connect(): Observable<Entries[]> {
    if (this.paginator) {
      this.paginator.length = this.length;
      if (this.pageSize !== undefined) {
        this.paginator.pageSize = this.pageSize;
      }
      if (this.pageIndex !== undefined) {
        this.paginator.pageIndex = this.pageIndex;
      }
    }
    if (this.sort) {
      if (this.activeSort !== undefined) {
        this.sort.active = this.activeSort;
      }
      if (this.sortDirection !== undefined) {
        this.sort.direction = this.sortDirection;
      }
    }
    return observableOf(this.data);
  }

  disconnect() {}
}
