import { Component, Inject, OnInit } from '@angular/core';
import { FormBuilder, FormControl, FormGroup } from '@angular/forms';
import { MatAutocompleteSelectedEvent } from '@angular/material/autocomplete';
import { MAT_DIALOG_DATA, MatDialogRef } from '@angular/material/dialog';
import { Observable } from 'rxjs';
import { debounceTime, distinctUntilChanged, startWith, switchMap } from 'rxjs/operators';

import { Batch } from '../core/batch';
import { BatchService } from '../core/batch.service';
import { Inventory } from '../core/inventory';
import { MathService } from '../shared/math.service';

@Component({
  selector: 'app-issue-dialog',
  templateUrl: './issue-dialog.component.html',
  styleUrls: ['./issue-dialog.component.css'],
})
export class IssueDialogComponent implements OnInit {
  batches: Observable<Batch[]>;
  form: FormGroup;
  batch: Batch = new Batch();

  constructor(
    public dialogRef: MatDialogRef<IssueDialogComponent>,
    @Inject(MAT_DIALOG_DATA) public data: { inventory: Inventory; date: string },
    private fb: FormBuilder,
    private math: MathService,
    private batchSer: BatchService,
  ) {
    this.form = this.fb.group({
      batch: '',
      quantity: '',
    });
    // Listen to Batch Autocomplete Change
    this.batches = (this.form.get('batch') as FormControl).valueChanges.pipe(
      startWith(''),
      debounceTime(150),
      distinctUntilChanged(),
      switchMap((x) => this.batchSer.autocomplete(this.data.date, x)),
    );
  }

  ngOnInit() {
    this.form.setValue({
      batch: this.data.inventory.batch,
      quantity: this.data.inventory.quantity,
    });
    this.batch = this.data.inventory.batch as Batch;
  }

  displayFn(batch?: Batch): string {
    return batch ? batch.name : '';
  }

  batchSelected(event: MatAutocompleteSelectedEvent): void {
    this.batch = event.option.value;
  }

  accept(): void {
    const formValue = this.form.value;
    const quantity = this.math.parseAmount(formValue.quantity, 2);
    this.data.inventory.batch = this.batch;
    this.data.inventory.quantity = quantity;
    this.data.inventory.rate = this.batch.rate;
    this.data.inventory.tax = this.batch.tax;
    this.data.inventory.discount = this.batch.discount;
    this.data.inventory.amount =
      quantity * this.batch.rate * (1 + this.batch.tax) * (1 - this.batch.discount);
    this.dialogRef.close(this.data.inventory);
  }
}
