import { Component, OnInit, ViewChild } from '@angular/core';
import { FormBuilder, FormGroup } from '@angular/forms';
import { MatPaginator } from '@angular/material/paginator';
import { MatSort } from '@angular/material/sort';
import { ActivatedRoute, Router } from '@angular/router';
import * as moment from 'moment';

import { ProfitLoss } from './profit-loss';
import { ProfitLossDataSource } from './profit-loss-datasource';

@Component({
  selector: 'app-profit-loss',
  templateUrl: './profit-loss.component.html',
  styleUrls: ['./profit-loss.component.css'],
})
export class ProfitLossComponent implements OnInit {
  @ViewChild(MatPaginator, { static: true }) paginator?: MatPaginator;
  @ViewChild(MatSort, { static: true }) sort?: MatSort;
  info: ProfitLoss = new ProfitLoss();
  dataSource: ProfitLossDataSource = new ProfitLossDataSource(this.info.body);
  form: FormGroup;
  selectedRowId = '';
  /** Columns displayed in the table. Columns IDs can be added, removed, or reordered. */
  displayedColumns = ['group', 'name', 'amount', 'total'];

  constructor(private route: ActivatedRoute, private router: Router, private fb: FormBuilder) {
    this.form = this.fb.group({
      startDate: '',
      finishDate: '',
    });
  }

  ngOnInit() {
    this.route.data.subscribe((value) => {
      const data = value as { info: ProfitLoss };

      this.info = data.info;
      this.form.setValue({
        startDate: moment(this.info.startDate, 'DD-MMM-YYYY').toDate(),
        finishDate: moment(this.info.finishDate, 'DD-MMM-YYYY').toDate(),
      });
      this.dataSource = new ProfitLossDataSource(this.info.body, this.paginator, this.sort);
    });
  }

  show() {
    const l = this.prepareSubmit();
    this.router.navigate(['profit-loss'], {
      queryParams: {
        startDate: l.startDate,
        finishDate: l.finishDate,
      },
    });
  }

  prepareSubmit(): ProfitLoss {
    const formModel = this.form.value;

    return new ProfitLoss({
      startDate: moment(formModel.startDate).format('DD-MMM-YYYY'),
      finishDate: moment(formModel.finishDate).format('DD-MMM-YYYY'),
    });
  }
}
