import { Component, OnInit, ViewChild } from '@angular/core';
import { FormBuilder, FormGroup } from '@angular/forms';
import { MatPaginator } from '@angular/material/paginator';
import { MatSort } from '@angular/material/sort';
import { ActivatedRoute, Router } from '@angular/router';
import * as moment from 'moment';

import { PurchaseEntries } from './purchase-entries';
import { PurchaseEntriesDataSource } from './purchase-entries-datasource';

@Component({
  selector: 'app-purchase-entries',
  templateUrl: './purchase-entries.component.html',
  styleUrls: ['./purchase-entries.component.css'],
})
export class PurchaseEntriesComponent implements OnInit {
  @ViewChild(MatPaginator, { static: true }) paginator?: MatPaginator;
  @ViewChild(MatSort, { static: true }) sort?: MatSort;
  info: PurchaseEntries = new PurchaseEntries();
  dataSource: PurchaseEntriesDataSource = new PurchaseEntriesDataSource(this.info.body);
  form: FormGroup;
  selectedRowId = '';
  /** Columns displayed in the table. Columns IDs can be added, removed, or reordered. */
  displayedColumns = [
    'date',
    'supplier',
    'product',
    'quantity',
    'rate',
    'tax',
    'discount',
    'amount',
  ];

  constructor(private route: ActivatedRoute, private router: Router, private fb: FormBuilder) {
    this.form = this.fb.group({
      startDate: '',
      finishDate: '',
    });
  }

  ngOnInit() {
    this.route.data.subscribe((value) => {
      const data = value as { info: PurchaseEntries };

      this.info = data.info;
      this.form.setValue({
        startDate: moment(this.info.startDate, 'DD-MMM-YYYY').toDate(),
        finishDate: moment(this.info.finishDate, 'DD-MMM-YYYY').toDate(),
      });
      this.dataSource = new PurchaseEntriesDataSource(this.info.body, this.paginator, this.sort);
    });
  }

  show() {
    const l = this.prepareSubmit();
    this.router.navigate(['purchase-entries'], {
      queryParams: {
        startDate: l.startDate,
        finishDate: l.finishDate,
      },
    });
  }

  prepareSubmit(): PurchaseEntries {
    const formModel = this.form.value;

    return {
      startDate: moment(formModel.startDate).format('DD-MMM-YYYY'),
      finishDate: moment(formModel.finishDate).format('DD-MMM-YYYY'),
      body: [],
    };
  }
}
