import { HttpClient } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { Observable } from 'rxjs/internal/Observable';
import { catchError } from 'rxjs/operators';

import { ErrorLoggerService } from '../core/error-logger.service';

import { StockMovement } from './stock-movement';

const url = '/api/stock-movement';
const serviceName = 'StockMovementService';

@Injectable({
  providedIn: 'root',
})
export class StockMovementService {
  constructor(private http: HttpClient, private log: ErrorLoggerService) {}

  list(startDate: string | null, finishDate: string | null): Observable<StockMovement> {
    const startDateWithSlash = startDate ? `/${startDate}` : '';
    const finishDateWishSlash = finishDate ? `/${finishDate}` : '';
    return this.http
      .get<StockMovement>(`${url}${startDateWithSlash}${finishDateWishSlash}`)
      .pipe(catchError(this.log.handleError(serviceName, 'list'))) as Observable<StockMovement>;
  }
}
